#pragma once

#include "wimp_basics.h"

#ifdef __cplusplus
extern "C"
{
#endif

  typedef enum
  {
    weNullReasonCode = 0,
    weRedrawWindowRequest,
    weOpenWindowRequest,
    weCloseWindowRequest,
    wePointerLeavingWindow,
    wePointerEnteringWindow,
    weMouseClick,
    weUserDragBox,
    weKeyPressed,
    weMenuSelection,
    weScrollRequest,
    weLoseCaret,
    weGainCaret,
    wePollwordNonZero,
    weUserMessage = 17,
    weUserMessageRecorded,
    weUserMessageAcknowledged
  } eWimpEvent; // we

  typedef enum
  {
    wmaMessageQuit = 0,
    wmaMessageDataSave,
    wmaMessageDataSaveAck,
    wmaMessageDataLoad,
    wmaMessageDataLoadAck,
    wmaMessageRun,
    wmaMessageRamFetch,
    wmaMessageRamTransmit,
    wmaMessagePrequit,
    wmaMessagePalette
  } eWimpMessageAction; // wma

  typedef struct
  {
    unsigned adjust : 1;
    unsigned menu : 1;
    unsigned select : 1;
    unsigned : 1;
    unsigned adjust_drag : 1;
    unsigned menu_drag : 1;
    unsigned select_drag : 1;
    unsigned : 1;
    unsigned adjust_click : 1;
    unsigned menu_click : 1;
    unsigned select_click : 1;
    unsigned : 1;
    unsigned : 16;
  } tButtonClick; // bc

  typedef struct
  {
    tWindowHandle whWindow;
    tLocation lMinimum;
    tLocation lMaximum;
    tLocation lScroll;
    tWindowHandle whBehind;
  } tWindowOpenBlock; // wob

  typedef struct
  {
    tWindowHandle whWindow;
  } tWindowOnlyBlock; // wob

  typedef struct
  {
    tLocation lMouse;
    tButtonClick bcButtons;
    tWindowHandle whWindow;
    tIconHandle ihIcon;
  } tMouseClickBlock; // mcb

  typedef struct
  {
    tLocation lMinimum;
    tLocation lMaximum;
  } tUserDrawBoxBlock; // udb

  typedef struct
  {
    tWindowHandle whWindow;
    tIconHandle ihIcon;
    tLocation lCaretOffset;
    unsigned int uiCaretHeightFlags;
    int iCaretIndex;
    int iCharacterCode;
  } tKeyPressedBlock; // kpb

  typedef struct
  {
    unsigned int iaSelectedItems[64];
  } tMenuSelectionBlock; // msb

  typedef struct
  {
    tWindowHandle whWindow;
    tLocation lMinimum;
    tLocation lMaximum;
    tLocation lScroll;
    tWindowHandle whBehind;
    tLocation lScrollDirection;
  } tScrollRequestBlock; // srb

  typedef struct
  {
    tWindowHandle whWindow;
    tIconHandle whIcon;
    tLocation lCaretOffset;
    unsigned int uiCaretHeightFlags;
    int iCaretIndex;
  } tLoseGainCaretBlock; // lgc

  typedef struct
  {
    int* piPollAddress;
    int iPollword;
  } tPollwordNonZeroBlock; // pnb

  typedef struct
  {
    int iBlockSize;
    tTaskHandle thSender;
    int iMyRef;
    int iYourRef;
    eWimpMessageAction wmaAction;
    unsigned int uiaInt[59];
    byte baByte[244];
  } tUserMessageBlock; // umb

  typedef union
  {
    tWindowOnlyBlock wobRedrawRequest;
    tWindowOpenBlock wobOpenRequest;
    tWindowOnlyBlock wobCloseRequest;
    tWindowOnlyBlock wobPointerLeaving;
    tWindowOnlyBlock wobPointerEntering;
    tMouseClickBlock mcbMouseClick;
    tUserDrawBoxBlock udbUserDrag;
    tKeyPressedBlock kpbKeyPressed;
    tMenuSelectionBlock msbMenuSelection;
    tScrollRequestBlock srbScrollRequest;
    tPollwordNonZeroBlock pnbPollwordNonZero;
    tLoseGainCaretBlock lgcLoseGainCaret;
    tUserMessageBlock umbUserMessage;

    byte baBytes[256];
    unsigned int uiaInt[64];
  } tWimpBlock;               // twb

  // Methods
  eWimpEvent wimp_poll(void);

  void wimp_sendmessage(eWimpEvent weEventCode, const tUserMessageBlock* cumbpBlock);

  void wimp_getpointerinfo(tMouseClickBlock* pmcbPointerInfo);

  // Objects
  extern tWimpBlock twbWimpPollBlock;

#ifdef __cplusplus
}
#endif
